@@ -14,16 +14,39 @@ from argparse import REMAINDER, SUPPRESS
 from clint.textui import colored, indent, puts, puts_err
 from continuity.services.commons import ServiceException
 from continuity.services.git import GitException, GitService
-from continuity.services.github import GitHubService, GitHubException
+from continuity.services.github import (GitHubException,
+        GitHubRequestException, GitHubService)
+from continuity.services.jira import JiraService
 from continuity.services.pt import PivotalTrackerService
 from continuity.services.utils import cached_property
-from getpass import getpass
 from os import chmod, rename
 from os.path import exists
 from requests.exceptions import ConnectionError
 from sys import exit
 
 
+MESSAGES = {
+    "continuity_integration_branch": "Integration branch",
+    "continuity_tracker": "Configure for (G)itHub Issues, (J)ira, or (P)ivotal Tracker?",  # NOQA
+    "git_branch": "Enter branch name",
+    "github_2fa_code": "GitHub two-factor authentication code",
+    "github_exclusive": "Exclude issues not assigned to you?",
+    "github_oauth_token": "GitHub OAuth token",
+    "github_password": "GitHub password",
+    "github_user": "GitHub user",
+    "jira_password": "JIRA password",
+    "jira_project_key": "JIRA project key",
+    "jira_url": "JIRA base url",
+    "jira_user": "JIRA username",
+    "pivotal_api_token": "Pivotal Tracker API token",
+    "pivotal_email": "Pivotal Tracker email",
+    "pivotal_exclusive": "Exclude stories which you do not own?",
+    "pivotal_password": "Pivotal Tracker password",
+    "pivotal_project_id": "Pivotal Tracker project ID"
+}
+MESSAGES["jira_exclusive"] = MESSAGES["github_exclusive"]
+
+
 class BaseCommand(object):
     """Base command.
 
@@ -204,20 +227,28 @@ class CommitCommand(BaseCommand):
 
                 if configuration:
                     continuity = git.get_configuration("continuity")
+                    tracker = continuity.get("tracker")
 
                     try:
-                        if continuity.get("tracker") == "github":
-                            number = configuration["issue"]
+                        if tracker == "jira":
+                            mention = configuration["issue"]
                         else:
-                            number = configuration["story"]
+                            if continuity.get("tracker") == "github":
+                                number = configuration["issue"]
+                            else:
+                                number = configuration["story"]
 
-                        mention = "#{0}".format(number)
+                            mention = "#{0}".format(number)
 
                         with open(self.namespace.file, 'r') as file:
                             message = file.read()
 
                         if mention not in message:
-                            message = "[{0}] {1}".format(mention, message)
+                            if tracker == "jira":
+                                message = "{0} #comment {1}".format(mention,
+                                    message)
+                            else:
+                                message = "[{0}] {1}".format(mention, message)
 
                             with open(self.namespace.file, 'w') as file:
                                 file.write(message)
@@ -303,6 +334,8 @@ class InitCommand(GitCommand):
         parser.add_argument("-n", "--new", action="store_true",
                 help="reinitialize from scratch")
         super(InitCommand, self).__init__(parser, namespace)
+        self.jira = {}
+        self.pivotal = {}
 
     def execute(self):
         """Execute this init command.
@@ -311,11 +344,12 @@ class InitCommand(GitCommand):
         puts()
         self.continuity = self.initialize_continuity()
 
-        if self.continuity["tracker"] == "pivotal":
+        if self.continuity["tracker"] == "jira":
+            puts()
+            self.jira = self.initialize_jira()
+        elif self.continuity["tracker"] == "pivotal":
             puts()
             self.pivotal = self.initialize_pivotal()
-        else:
-            self.pivotal = {}
 
         puts()
         self.github = self.initialize_github()
@@ -329,6 +363,7 @@ class InitCommand(GitCommand):
 
         self.git.set_configuration("continuity", **self.continuity)
         self.git.set_configuration("github", **self.github)
+        self.git.set_configuration("jira", **self.jira)
         self.git.set_configuration("pivotal", **self.pivotal)
         self.git.set_configuration("alias", **self.aliases)
 
@@ -368,6 +403,9 @@ class InitCommand(GitCommand):
         puts("Configured git for continuity:")
 
         with indent():
+            for key, value in self.jira.iteritems():
+                puts("jira.{0}={1}".format(key, value))
+
             for key, value in self.pivotal.iteritems():
                 puts("pivotal.{0}={1}".format(key, value))
 
@@ -396,24 +434,21 @@ class InitCommand(GitCommand):
             configuration = self.git.get_configuration("continuity")
 
         branch = configuration.get("integration-branch", self.branch.name)
-        branch = prompt("Integration branch", branch)
+        branch = prompt(MESSAGES["continuity_integration_branch"], branch)
         tracker = configuration.get("tracker")
-        tracker = prompt("Configure for (G)itHub Issues or (P)ivotal Tracker?",
-                tracker, characters="GP")
+        tracker = prompt(MESSAGES["continuity_tracker"], tracker,
+                characters="GJP", echo=False)
 
         if tracker == 'G':
             tracker = "github"
+        elif tracker == 'J':
+            tracker = "jira"
         elif tracker == 'P':
             tracker = "pivotal"
 
-        exclusive = configuration.get("exclusive", False)
-
-        if tracker == "github":
-            exclusive = confirm("Exclude issues not assigned to you?",
-                    default=exclusive)
-        else:
-            exclusive = confirm("Exclude stories which you do not own?",
-                    default=exclusive)
+        message = MESSAGES["{0}_exclusive".format(tracker)]
+        default = configuration.get("exclusive", False)
+        exclusive = confirm(message, default=default)
 
         return {
             "integration-branch": branch,
@@ -428,19 +463,74 @@ class InitCommand(GitCommand):
         token = configuration.get("oauth-token")
 
         if token and not self.namespace.new:
-            token = prompt("GitHub OAuth token", token)
+            token = prompt(MESSAGES["github_oauth_token"], token)
         else:
-            user = prompt("GitHub user", configuration.get("user"))
-            password = getpass("GitHub password: ")
+            user = prompt(MESSAGES["github_user"], configuration.get("user"))
+            password = prompt(MESSAGES["github_password"], echo=False)
             name = "continuity:{0}".format(self.git.repo.working_dir)
             url = "https://github.com/jzempel/continuity"
-            token = GitHubService.create_token(user, password, name, url)
+
+            try:
+                token = GitHubService.create_token(user, password, name,
+                        url=url)
+            except GitHubRequestException:
+                code = prompt(MESSAGES["github_2fa_code"])
+                token = GitHubService.create_token(user, password, name,
+                        code=code, url=url)
 
             if not token:
                 exit("Invalid GitHub credentials.")
 
         return {"oauth-token": token}
 
+    def initialize_jira(self):
+        """Initialize jira data.
+        """
+        if self.namespace.new:
+            configuration = {}
+        else:
+            configuration = self.git.get_configuration("jira")
+
+        url = prompt(MESSAGES["jira_url"], configuration.get("url"))
+        token = configuration.get("auth-token")
+        project_key = configuration.get("project-key")
+        user = prompt(MESSAGES["jira_user"], configuration.get("user"))
+
+        if not token or user != configuration.get("user"):
+            password = prompt(MESSAGES["jira_password"], echo=False)
+            token = JiraService.get_token(user, password)
+
+        jira = JiraService(url, token)
+
+        try:
+            jira.get_user()
+        except:
+            exit("Invalid JIRA credentials.")
+
+        projects = jira.projects
+
+        if projects:
+            if not project_key:
+                for project in projects:
+                    message = "{0} - {1}".format(colored.yellow(project.key),
+                            project.name)
+                    puts(message)
+
+            project_key = prompt(MESSAGES["jira_project_key"], project_key)
+            project = jira.get_project(project_key)
+
+            if not project:
+                exit("Invalid project key.")
+        else:
+            exit("No JIRA projects found.")
+
+        return {
+            "auth-token": token,
+            "project-key": project_key,
+            "url": url,
+            "user": user
+        }
+
     def initialize_pivotal(self):
         """Initialize pivotal data.
         """
@@ -451,14 +541,15 @@ class InitCommand(GitCommand):
 
         token = configuration.get("api-token")
         project_id = configuration.get("project-id")
-        email = configuration.get("email")
+        email = configuration.get("email",
+                self.git.get_configuration("user").get("email"))
         owner_id = configuration.get("owner-id")
 
         if token:
-            token = prompt("Pivotal Tracker API token", token)
+            token = prompt(MESSAGES["pivotal_api_token"], token)
         else:
-            email = prompt("Pivotal Tracker email", email)
-            password = getpass("Pivotal Tracker password: ")
+            email = prompt(MESSAGES["pivotal_email"], email)
+            password = prompt(MESSAGES["pivotal_password"], echo=False)
             token = PivotalTrackerService.get_token(email, password)
 
             if not token:
@@ -474,14 +565,14 @@ class InitCommand(GitCommand):
                             project.name)
                     puts(message)
 
-            project_id = prompt("Pivotal Tracker project ID", project_id)
+            project_id = prompt(MESSAGES["pivotal_project_id"], project_id)
             project = pt.get_project(project_id)
 
             if project:
                 try:
                     password
                 except NameError:
-                    email = prompt("Pivotal Tracker email", email)
+                    email = prompt(MESSAGES["pivotal_email"], email)
 
                 for member in project.members:
                     if member.email == email:
@@ -532,7 +623,7 @@ class ReviewCommand(GitHubCommand):
             for error in getattr(e, "json").get("errors", []):
                 if error["code"] == "custom":
                     error_message = error["message"]
-                    message = "{0} - {1}{2}.".format(message,
+                    message = "{0} - {1}{2}".format(message,
                             error_message[:1].lower(), error_message[1:])
                     break
             else:
@@ -567,7 +658,7 @@ class StartCommand(GitCommand):
         branch = self.get_value("continuity", "integration-branch")
 
         if self.namespace.force or branch == self.branch.name:
-            name = prompt("Enter branch name")
+            name = prompt(MESSAGES["git_branch"])
             ret_val = '-'.join(name.split())
 
             try:
@@ -579,6 +670,10 @@ class StartCommand(GitCommand):
 
                 puts("Switched to a new branch '{0}'".format(ret_val))
             except GitException, e:
+                if not self.git.remote:
+                    self.git.get_branch(branch)
+                    self.git.delete_branch(ret_val)
+
                 exit(e)
         else:
             message = "error: Attempted start from non-integration branch; switch to '{0}'.\nUse -f if you really want to start from '{1}'."  # NOQA
@@ -647,12 +742,24 @@ def get_commands(tracker=None):
 
         if not tracker:
             continuity = git.get_configuration("continuity")
-            tracker = continuity.get("tracker")
+            tracker = continuity["tracker"]
 
         if tracker == "github":
             from .github import (FinishCommand, IssueCommand, IssuesCommand,
                     ReviewCommand, StartCommand, TasksCommand)
 
+            ret_val.update({
+                FinishCommand.name: FinishCommand,
+                IssueCommand.name: IssueCommand,
+                IssuesCommand.name: IssuesCommand,
+                ReviewCommand.name: ReviewCommand,
+                StartCommand.name: StartCommand,
+                TasksCommand.name: TasksCommand
+            })
+        elif tracker == "jira":
+            from .jira import (FinishCommand, IssueCommand, IssuesCommand,
+                    ReviewCommand, StartCommand, TasksCommand)
+
             ret_val.update({
                 FinishCommand.name: FinishCommand,
                 IssueCommand.name: IssueCommand,
@@ -673,7 +780,7 @@ def get_commands(tracker=None):
                 StartCommand.name: StartCommand,
                 TasksCommand.name: TasksCommand
             })
-    except GitException:
+    except (GitException, KeyError):
         from .commons import (FinishCommand, ReviewCommand, StartCommand,
                 TasksCommand)
 
